/******************************************************************************
;  *       @型号                 : MC51F7424
;  *       @创建日期             : 2022.12.22
;  *       @公司/作者            : SINOMCU.COM-AE
;  *       @晟矽微官网           : http://www.sinomcu.com/
;  *       @版权                 : 2022 晟矽微电 版权所有.
;  *----------------------摘要描述---------------------------------
;  *LIN slalve,receive master sand answer data and dispose message,send
;  *back slave answer data.
******************************************************************************/

#include "User_Def.h"

extern uint8_t LIN_Recv_buff[]; 
extern uint8_t LIN_Send_buff[]; 
LIN_Frame LIN_slave_frame = {UNCONDITOPMAL_FRAME,NO_ERROR,RESET,LIN_Recv_buff};
LINStatus LIN_bus_stauts =BUS_IDEL;

#define UART0_TransmitData8(tx_data)     SBUF = tx_data; \
                                         while (!Read_Uart0_TI()) { ;}  \
                                         Uart0_TI_clear()

/**
  * @brief initial lin arguments
  * @param  
  * @retval
  */ 
void LinInit(void) {
  LIN_slave_frame.lin_frame_type =UNCONDITOPMAL_FRAME;
  LIN_slave_frame.lin_frame_error=NO_ERROR;
  LIN_slave_frame.lin_dispose_requset =RESET;
  LIN_slave_frame.recv_buff =LIN_Recv_buff;
  LIN_bus_stauts =BUS_IDEL;
}

/**
  * @brief process reviced answer data and check PID,compare checksum
  * @param  
  * @retval
  */
void LIN_RecvDataProcess(void) {
  uint8_t id=0,pid=0;
  if( LIN_slave_frame.lin_dispose_requset ==SET) {    /*recive new frame*/
    LIN_slave_frame.lin_dispose_requset =RESET;
    pid=LIN_slave_frame.recv_buff[0];
    id=GET_ID(pid);
    if(id == LIN_RECV_ANSWER_FID) {                   /*FID need dispose*/
      if( LIN_slave_frame.lin_frame_error == SYSNC_ERROR) {          /*check sysnc error*/
        printf("--Error:Frame Error:Sysnc error!\r\n");
      }
      if( LIN_CheckPID(pid) ==RESET) {                               /*check pid error*/
        LIN_slave_frame.lin_frame_error |= PID_ERROR;
        printf("--Error:Frame Error:PID error!\r\n");
      }

      if( LIN_CompareChecksum(LIN_slave_frame.recv_buff ) ==RESET) { /*chechsum error*/
        LIN_slave_frame.lin_frame_error |= CHECKSUM_ERROR;
        printf("--Error:Frame Error:Checksum error!\r\n");
      }

      if( LIN_slave_frame.lin_frame_error == NO_ERROR) {             /*fram correct,data dispose*/
        if( LIN_slave_frame.recv_buff[LIN_RECV_DATA_POS] ==LIN_RECV_DATA_ID) {
          LED1_TOGGLE();
          printf("--Inf:LED1 Toggle.\r\n");
        }
      }else {
        /*frame error*/        
        LIN_slave_frame.lin_frame_error = NO_ERROR;
      }
    }
  }
}

/**
  * @brief updata send buff
  * @param  
  * @retval
  */
void Upata_SendBuff(void) {
  if(Get_LED1Status()==SET) {     /*SET:LED ON*/  
    LIN_Send_buff[LIN_SEND_DATA_POS] =LED_ON_SEND_DATA;
  }else{                          /*RESET:LED OFF*/
    LIN_Send_buff[LIN_SEND_DATA_POS] =LED_OFF_SEND_DATA;   
  }
}

/**
  * @brief send answer data
  * @param  
  * @retval
  */
void LIN_SendAnswer(void) {
  uint8_t send_count=0,checksum=0;

  for(send_count=0;send_count<LIN_FRAM_DATA_LEN ;send_count++) {     /*send answer data*/
    UART0_TransmitData8(LIN_Send_buff[send_count]);
  }

  checksum =LIN_CaclChecksum(LIN_CaclPID(LIN_SEND_ANSWER_FID),LIN_Send_buff);
  UART0_TransmitData8(checksum);       /*send check sum*/
}

/**
  * @brief  Function called in UART IT function
  * @param  
  * @retval 
  */
void UART_RI_Callback(void) {
  uint8_t recv_char=0;
  static uint8_t recv_count=0;

  recv_char = SBUF;

  switch(LIN_bus_stauts) {
    case BUS_IDEL:               /* interval segment*/
      if((recv_char ==0x00) &&(Read_Uart0_FE())){
        LIN_bus_stauts =LIN_SYSNC;
        Uart0_FE_clear();
      }
      break;
    case LIN_SYSNC:               /*sysnc segment*/
      if(recv_char ==LIN_SYNC_CHAR) {
        LIN_bus_stauts =LIN_PID;
        recv_count=0;
      }else{ 
        LIN_slave_frame.lin_frame_error =SYSNC_ERROR;
      }
      break;
    case LIN_PID:                 /*PID segment*/
      if( LIN_RECV_ANSWER_FID == GET_ID(recv_char)) {      /*master send answer data*/
        LIN_slave_frame.recv_buff[recv_count++] =recv_char;     /*save PID to receive buff*/
        LIN_bus_stauts =  LIN_ANSWER;        
      } else if(LIN_SEND_ANSWER_FID ==GET_ID(recv_char)) { /*need slave send answer data*/
        LIN_SendAnswer();
        LIN_bus_stauts =BUS_IDEL;
      } else {                                             /*other slave send answer data*/
        LIN_bus_stauts =BUS_IDEL;
      }
      break;      
    case LIN_ANSWER:             /*receive master send answer data*/
      LIN_slave_frame.recv_buff[recv_count++] =recv_char;
      if(recv_count ==LIN_RECV_DATA_LEN) {                  /*frame end*/
        LIN_bus_stauts =BUS_IDEL;
        LIN_slave_frame.lin_dispose_requset =SET;
      }
      break;      
    default :
      LIN_bus_stauts =BUS_IDEL;
  }
}

/******************************** END OF FILE *********************************/